#!/usr/bin/python
#
# This script will take a running process ID and name of application
# to construct SYSTEMD style init files, the output files are
# <appname> and <appname>.service should be copied to target machines 
# /etc/sysconfig/<appname>  and /etc/systemd/system/<appname>.service
#
#
import  sys

pidstr = sys.argv[1]
appname = sys.argv[2]

if not pidstr.isdigit() :
    print >> sys.stderr, "the first parameter is not a process ID"
    sys.exit(1)

envFileLine = ""

try:
    inFile = open("/proc/" + pidstr + "/environ", "r", 16000)
    readbuffer = inFile.read();
    if not readbuffer:
        print >> sys.stderr, "no environ file"
        inFile.close()
    else:
        if len(readbuffer) < 3:
            print >> sys.stderr, "Empty environ file" 
            inFile.close()
        else:
            outFile = open(appname, "w") 
            alllines = readbuffer.split('\0');
            for oneline in alllines:
                if len(oneline) < 3:
                    continue
                outFile.write(oneline + "\n")
        
            inFile.close()
            outFile.flush()
            outFile.close()
        
            envFileLine = "EnvironmentFile=-/etc/sysconfig/" + appname

    inFile = open("/proc/" + pidstr + "/cmdline", "r", 16000)
    readbuffer = inFile.read()
    if not readbuffer:
        print >> sys.stderr, "no cmdline file"
        sys.exit(1)

    outFile = open(appname + ".service", "w")
    outFile.write("[Unit]\n##  You may modify the Description below\nDescription=" + appname +"\n")
    outFile.write("##   Below can be used to define service dependencies, for example, to tell system that this service is after ntpd and sshd\n");
    outFile.write("#After=ntpd.service sshd.service\n#Requires=ntpd.service sshd.service\n\n");
    outFile.write("[Service]\n##  If your program does not fork into background, comment out below\n")
    outFile.write("Type=forking\n#\n##  You may add User/Group/PIDFile/ExecStartPre/ExecStop/UMask, examples below\n")
    outFile.write("#User=myuser\n#Group=mygroup\n#PIDFile=/var/run/"+appname+".pid\n#UMask=0002\n")
    outFile.write("#ExecStartPre=/bin/mkdir -p /tmp/workdir/"+appname+"\n#ExecStartPre=/bin/chown myuser /tmp/workdir/" + appname +"\n#\n")
    if len(envFileLine) > 5:
        outFile.write(envFileLine + "\n");
    outFile.write("ExecStart=")
    allparams = readbuffer.split('\0');
    for oneparam in allparams:
        outFile.write(oneparam + " ")
 
    outFile.write("\n\n[Install]\nWantedBy=multi-user.target\n")
    inFile.close()
    outFile.flush()
    outFile.close()

    if len(envFileLine) > 5:
        print "\n  Successfully created files " + appname + " and " + appname + ".service"
        print "    1.  Copy " + appname  + " to /etc/sysconfig/" + appname
        print "    2.  Verify file " + appname + ".service  and then copy it to /etc/systemd/system/" + appname + ".service"
        print "    3.  Start systemd service:  systemctl start " + appname + "\n"

except OSError:
    print >> sys.stderr, "Other OS error "
    sys.exit(1)
except IOError:
    print >> sys.stderr, "Could not find environ file, invalid process ID"
    sys.exit(1)


